#==============================================================================
# §package      DE_EPAGES::Content::UI::ContentObject
# §state        private
#==============================================================================
package DE_EPAGES::Content::UI::ContentObject;
use base DE_EPAGES::MediaGallery::UI::Object;

use DE_EPAGES::Content::API::Constants qw ( CATEGORY_IMAGES_SIZES CATEGORY_IMAGE_SIZE_ABBREVIATION MAX_NESTING_OF_CATEGORIES);
use DE_EPAGES::Content::API::ShortUrl qw (GetShortUrlByClass);
use DE_EPAGES::Core::API::Error qw( Error GetError ExistsError);
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile fsplit RelativePath );
use DE_EPAGES::Core::API::Image qw ( CorrectImage ScaleImage ResizeImageUsingImageRef TestImageFormat TestImage );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Alias Unicode2Filename );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( Transaction LoadClassByAlias );
use DE_EPAGES::Presentation::API::Object::PagerChildren;
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByAlias);
use DE_EPAGES::Presentation::API::Upload qw ( GetJSONImagesInfo );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use strict;



sub ErrorDefinition {
    return {
        'WrongParent' => 'Invalid parent id: #ObjectParentID. Expected: #ParentID',
        'WrongClass' => 'Invalid class id: #ClassID',
    };
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $Category = $Servlet->object;

   # check if we have a object name only with special chars
    my $ObjectName = $Servlet->form->value('Name');
    if (defined $ObjectName && $ObjectName =~ /\S/){
        $Servlet->form->addFormError({
            'Reason' => 'NAME_NOT_VALID',
            'Name'   => 'Name',
            'Value'  => $ObjectName,
        }) unless $ObjectName =~ /\w/;
    }

    my %IndexMap;
    my $Index = 0;
    my @LanguageIDs = $Servlet->form->value('LanguageID');
    foreach my $LanguageID (@LanguageIDs) {
        $IndexMap{$LanguageID} = $Index unless defined $IndexMap{$LanguageID};;
        $Index++;
    }
    # save the main content of the category
    eval {
        RunBatchProcess('UpdateCategories', sub {
            $self->SUPER::Save($Servlet);
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ($Error->code eq 'INVALID_CHARACTER' ||
                              $Error->code eq 'SHORTURL_ALREADY_EXISTS' ||
                              $Error->code eq 'ShortURL_NOT_ALLOWED');
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'LoopName' => 'Save',
            'Name'  => 'URI',
            'Index' => $IndexMap{$Error->vars->{'LanguageID'}},
        });
    }

    # scale Image
    if ($Servlet->params('ScaleImageSize') and $Servlet->params('ScaleImageSize') > 1 and $Servlet->params('ImageMedium')) {
        my $FileName = $Category->get('PublicPath').'/'. $Category->get('ImageMedium');
        GetLog->debug("ScaleImage($FileName, $FileName, ".$Servlet->params('ScaleImageSize').')');

        eval {
            ScaleImage($FileName, $FileName, $Servlet->params('ScaleImageSize')) if ExistsFile($FileName);
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'IMAGE_MAGICK' ||
                                  $Error->code eq 'UNSUPPORTED_IMAGE_FORMAT');
            $Servlet->form->executeFormError({
                'Reason'=> $Error->code,
                'Form'  => 'Save',
                'Name'  => 'ImageMedium',
            });
        }
    }

    # throw changed-hook
    TriggerHook('UI_AfterSaveContentObject', {'Object' => $Category});

    return;
}

sub SaveUnityStartPage {
    my $self = shift;
    my ($Servlet) = @_;

    # check form values
    my $StartPage = $Servlet->object;
    $Servlet->form->form($StartPage, 'UnityStartPage');

    $self->Save($Servlet);

    return;
}

sub DeleteImage {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params('DeleteImage',1);
    return unless $Servlet->params('AcceptDeleteImage');

    my $Object = $Servlet->object;
    my $FileName = $Object->get('PublicPath').'/'.$Object->get('ImageMedium');
    $Object->set({'ImageMedium' => undef});

    DeleteFile($FileName);
    $Servlet->params('DeleteImage',undef);
    return;
}

sub SetVisibleInNavigation {
    my $self = shift;
    my ($Servlet) = @_;
    return $self->SetAttributeOnListedObjects($Servlet, 'VisibleInNavigation', 1);
}

sub SetNotVisibleInNavigation {
    my $self = shift;
    my ($Servlet) = @_;
    return $self->SetAttributeOnListedObjects($Servlet, 'VisibleInNavigation', 0);
}

sub ViewTag {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $ContentObject = $Servlet->object;
    $self->SUPER::View($Servlet);

    my ($Tag) = $Form->value('Tag');
    Error('OBJECT_NOT_FOUND', { ObjectID => $ContentObject->id }) unless defined $Tag;

    my $ClassContentTag = LoadClassByAlias('ContentTag');
    my $aContentTags = $ClassContentTag->getObjects('ByContentObjectIDTag', [$ContentObject->id, $Tag]);
    return unless scalar @$aContentTags;
    $Servlet->vars( 'Tag', $aContentTags->[0]->tleHash);
    return;
}

sub SaveList {
    my $self = shift;
    my ($Servlet) = @_;

    my $Parent = $Servlet->object;

    #need for getting Position after $Parent->renumberChildren
    my $TempObject=undef;
    eval {
        if ($Servlet->form->value('Alias')) {
            $TempObject = $self->NewObject( $Servlet ) ;
        }
        elsif ($Servlet->form->value('Name')){
            my $Alias = $Servlet->form->value('Name');
            my $Class = $Parent->getSite()->loadObject($Servlet->form->value('ClassID'), 'Class', {'AllowSystemSite' => 1});
            my $URI = GetShortUrlByClass($Parent, $Class, $Alias);
            $Alias = Unicode2Alias($Alias);
            $TempObject = $self->NewObject($Servlet, undef, undef, {Alias=>$Alias, URI=>$URI});
            $TempObject->set( { Position => $Servlet->form->value('NewPosition') } );
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->mapFormError({'Name'=>'ClassID'},$Error);
        $Servlet->form->mapFormError({'Name'=>'Alias'},$Error);
        if ($Error->code eq 'OBJECT_EXISTS'){
            $Servlet->vars('OBJECT_EXISTS',1);
        }
        elsif ($Error->code =~ /FeatureLimitExceededSubcategories/){
            $Servlet->vars('FeatureMaxValueReachedSubcategories',1);
        }
        elsif ($Error->code  eq 'FeatureLimitExceeded') {
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }
        else {
            Error($Error);
        }
    }
    $Servlet->form->executeFormError;

    my $ahListItems = $self->listItems( $Servlet, $Parent, 'ContentObject' );

    foreach my $hListItem (@$ahListItems) {
        my $Position = $hListItem->{'Position'};
        my $Object = $hListItem->{'Object'};

        # security check - verify that the object is a child of given parent
        if( $Object->parent->id != $Parent->id ) {
            Error( 'WrongParent', {
                ObjectParentID => $Object->parent->id,
                ParentID => $Parent->id
            } );
        }
        $Object->set( { Position => $Position } );
    }
    $Parent->renumberChildren;
    $Parent->updatePageCache;

    #calculate position in pager
    if (defined $TempObject) {

        #register pageposition in servlet
        $Servlet->vars('JumpToObjectID',$TempObject->id);
    }
    return;
}

sub ViewMoveTo {
    my $self = shift;
    my ($Servlet) = @_;

    return unless $Servlet->params('ListObjectID');

    my @ListObjectIDs = $Servlet->form->value('ListObjectID');
    my $Shop = $Servlet->object()->getSite();
    my @ObjectsToMove;
    foreach my $ListObjectID (@ListObjectIDs) {

        my $ListObject = $Shop->loadObject($ListObjectID, 'ContentObject');
        $Servlet->testPermission($ListObject, 'Save');

        my $hValues;
        $hValues->{'ListObjectParentID'} = $ListObject->parent->id;
        $hValues->{'ListObjectID'} = $ListObjectID;
        push @ObjectsToMove ,$hValues;
    }
    $Servlet->vars( 'ObjectsToMove', \@ObjectsToMove );
    return;
}

sub MoveTo {
    my $self = shift;
    my ($Servlet) = @_;

    my $CategoryID = $Servlet->params('CategoryID');
    my $MasterObject = $Servlet->object;
    my $Shop = $MasterObject->getSite();
    my $IsUnityShop = $Shop->get('IsUnityRegistered');

    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'ContentObject' ) }
                      @{$hValues->{'ListObjectIDs'}};
    $Servlet->testPermission($_, 'Save') foreach @ListObjects;

    # test if alias already exists in target category
    # and if the parent allows that kind of children
    my $Parent = $Shop->loadObject($CategoryID, 'Category');
    $Servlet->testPermission($Parent, 'Save');

    my $ParentCategoryLevel;
    $ParentCategoryLevel = $Parent->countCategoryNesting if $IsUnityShop;

    foreach my $Index ( 0 .. scalar(@ListObjects)-1 ) {
        my $Object = $ListObjects[$Index];

        # check class
        next if !$self->checkRequiredClass($Form, $Parent, $Object->class);

        # check alias
        my $Alias = $Object->alias;
        if ($Parent->findChild($Alias)) {
            $Servlet->vars('ViewAction','MBO-ViewMoveTo');
            $Servlet->vars('ALIAS_EXISTS',1);
            $Form->addFormError( {
                'Reason' => 'ALIAS_EXISTS',
                'Index' => $Index,
                'LoopName' => 'ListObjectIDs',
                'Form' => 'ListedObjects',
                'Name' => 'ListObjectID',
            } );
        }

        # check for the maximum nesting of categories in case of a unity shop
        if($IsUnityShop) {
            if($ParentCategoryLevel + $Object->countChildrenDepth >= MAX_NESTING_OF_CATEGORIES) {
                $Servlet->vars('TooManyLevelsOfCategories', {
                    MaxValue    => MAX_NESTING_OF_CATEGORIES
                });
                $Form->addFormError( {
                    'Reason'  => 'TooManyLevelsOfCategories',
                    'Name'    => $Object->get('NameOrAlias')
                });
            }
        }
    }
    $Form->executeFormError;

    RunBatchProcess('UpdateCategories', sub {
        Transaction( sub {
            foreach my $ListObject (@ListObjects) {
                if ($ListObject->id != $Parent->id) {
                    $ListObject->set( { 'Parent' => $Parent } );
                } else {
                    GetLog('BUGTRACKER::AD7050')->debug('Prevented object from setting itself as parent: '.$ListObject->id);
                }
            }
            $Parent->renumberChildren;
        });
    });

    $Parent->getSite()->invalidatePageCache;
    return;
}

# change a ChangeAction to ViewAction for the Batchaction "Move To..."
sub ChangeActionToViewAction {
    my $self = shift;
    my ($Servlet) = @_;
    return unless $Servlet->params('ListObjectID');
    $Servlet->vars('ViewAction',$Servlet->params('MoveTo_ViewAction'));
    return;
}

sub CloneElements {
    my $self = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my $Shop = $MasterObject->getSite();

    my @ListObjects;
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'ContentObject');
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    my $ahShopLanguages = $MasterObject->getSite()->get('Languages');

    # needed for error message
    my $LastObject;
    eval {
        RunBatchProcess(['CreateObjects', 'CreateCategories'], sub {
            Transaction( sub {
                foreach my $Object (@ListObjects) {

                    if (not $Object->canClone()) {
                        $Form->addFormError( {
                            'Reason'        => 'CantCloneContentObject',
                            'LoopName'      => 'ListObjectIDs',
                            'Form'          => 'ListedObjects',
                            'Name'          => 'ListObjectID',
                            'ContentObject' => $Object->tleHash,
                        } );
                        next;
                    }

                    $LastObject = $Object;
                    my $NewAlias = $Object->alias;
                    my $cnt = 1;

                    # test if Alias has already an ext like [1] and replace ext if exists
                    if ($NewAlias =~ /.*\[\d{1,}\]$/gx) {
                        while ($Object->parent->existsChild($NewAlias)) {
                            $cnt++;
                            $NewAlias =~ s/(\[)\d{1,}(\])$/$1$cnt$2/gx;
                        }
                    }
                    else {
                        while ($Object->parent->existsChild($NewAlias."[$cnt]")){
                            $cnt++;
                        }
                        $NewAlias .= "[$cnt]";
                    }


                    # clone the object and set cloned object invisible
                    my $NewObject = $Object->clone({'Alias'=> $NewAlias, 'Position'=> $Object->get('Position')+1, 'Inherit' => 1,'IsVisible' => 0},1);
                    $Object->parent->renumberChildren;

                    # update name
                    foreach my $hShopLanguage (@$ahShopLanguages) {
                        my $NewName = $Object->get('Name', $hShopLanguage->{'LanguageID'});
                        next unless defined $NewName;
                        next if $NewName eq '';
                        if ($NewName =~ /.*\[\d{1,}\]$/gx) {
                            $NewName =~ s/(\[)\d{1,}(\])$/$1$cnt$2/gx;
                        }
                        else {
                            $NewName .= "[$cnt]";
                        }
                        $NewObject->set({'Name' => $NewName}, $hShopLanguage->{'LanguageID'});
                    }
                }
            });
        });
    };

    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
            Warning($Error);
        } elsif ($Error->code  eq 'OBJECT_EXISTS'){
            $Servlet->form->executeFormError({
                'Reason'=> 'CloneUniqueObject',
                'Name' => 'DUMMY',
                'Form' => 'DUMMY',
                'Vars' => {
                    'Object' => $LastObject->tleHash
                }
            });
        } else {
            Error($Error);
        }
    }

    $Form->executeFormError;

    return;
}

#==============================================================================
# §function     SetView
# §syntax       $ContentObject->SetView($Servlet);
# §description  loads a new image for given attribute
# §input        Servlet | image attribute alias | ref.hash
#==============================================================================
sub SetView {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $Form = $Servlet->form;

    my $FilePath = $Form->value('NewFile');
    my (undef, $FName, $Extension) = fsplit($FilePath);

    return unless defined $FilePath;  # file input field does not exist in form
    return unless $FilePath =~ /\S/;  # no file name was submitted

    my $Attribute = $Form->value('Image');
    Error('NoImageAttribute') unless defined $Attribute and $Attribute =~ /\S/;

    my $Size = CATEGORY_IMAGE_SIZE_ABBREVIATION->{$Attribute};
    my $Value = Unicode2Filename($FName).($Size eq 'l' ? '': '_'.$Size).$Extension;

    # rewrite form value for file name
    $Form->{'Values'}->{'NewFile'}->[0] = $Value;

    $self->UploadImages($Servlet, {'Index' => 0, 'Name' => 'NewFile'} );

    $Object->set({ $Attribute => $Servlet->vars('ResponseContent')->{'uploadedObjectFile'} });
    $Object->getSite()->invalidatePageCache;

    # throw changed-hook
    TriggerHook('UI_AfterSaveContentObject', {'Object' => $Object});

    return;
}

sub ExportSelectedObjects{
    my $self = shift;
    my ($Servlet) = @_;

    my $ContentObject = $Servlet->object;
    my $hValues = $Servlet->form->form($ContentObject, 'ListedObjects');
    my $Shop = $ContentObject->getSite();

    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'ContentObject' ) } @{$hValues->{'ListObjectIDs'}};
    return unless scalar @ListObjects;

    $Servlet->vars('SelectedObjects', [map {$_->tleHash} @ListObjects]);
    $Servlet->vars('CSVExportImportType', 'Product');
    $Servlet->vars('EXPORT_SELECTED_OBJECTS', 1);
    $Servlet->vars('ViewActionAbort', $Servlet->params('ViewAction'));
    $Servlet->vars('ObjectIDAbort', $ContentObject->id);
    $Servlet->vars('ViewAction', 'MBO-ViewCSVExportImportContent');
    $Servlet->vars('ViewObjectID', $ContentObject->getSite->id);
    return;
}

sub NewObject {
    my $self = shift;
    my ($Servlet, @Parameter) = @_;

    my $Parent = $Servlet->object;
    my $Class = $Parent->getSite()->loadObject($Servlet->form->value('ClassID'), 'Class', {'AllowSystemSite' => 1});

   # check if we have a object name only with special chars
    my $ObjectName = $Servlet->form->value('Name');
    if (defined $ObjectName && $ObjectName =~ /\S/){
    $Servlet->form->addFormError({
            'Reason' => 'NAME_NOT_VALID',
            'Name'   => 'Name',
            'Value'  => $ObjectName,
        }) unless $ObjectName =~ /\w/;
    }

    my $URI = GetShortUrlByClass($Parent, $Class, $Servlet->form->value('Name'));

    my $TempObject;
    RunBatchProcess('CreateCategories', sub {
        $TempObject = $self->SUPER::NewObject($Servlet, $Parent, $Class, { 'URI' => $URI });
    });

    if($TempObject->instanceOf('Category') && !$TempObject->instanceOf('StartPage')) {
        my $Category = $TempObject;
        my $PageType = LoadPageTypeByAlias( 'SF-Category' );

        foreach my $TemplateFolder (@{$PageType->templateTypeFolders}) {
            my $TemplateName = $TemplateFolder->alias;
            my $TemplateTypeName = $PageType->templateTypeAlias( $Parent, $TemplateName );

            next if (not defined $TemplateTypeName) or ($TemplateName eq $TemplateTypeName);
            $PageType->setTemplateType( $Category, $TemplateName, $TemplateTypeName );
        }
    }

    # trigger hook so other classes can set defaults etc.
    TriggerHook('UI_AfterNewContentObject', {'Object' => $TempObject});

    return $TempObject;
}

#========================================================================================
# §function     ViewPagedChildren
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hVars = $Handler->ViewPagedChildren( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Creates a pager for child objects of $Servlet->object.
#               The default sort order is by Position, ascending.
#----------------------------------------------------------------------------------------
# §input        $Servlet | current servlet (contains form data) | object
# §return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub ViewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    my $Pager = DE_EPAGES::Presentation::API::Object::PagerChildren->new({
        'Servlet' => $Servlet,
        'Object' => $Servlet->object
    });

    if (defined $Servlet->params('JumpToObjectID')){
        my $Page = $Pager->pageOfItemByObjectID($Servlet->params('JumpToObjectID'));
        $Pager->set({'Page' => $Page}) if defined $Page;
    }

    return $Pager->page;
}

#========================================================================================
# §function     UnityViewPagedChildren
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hVars = $Handler->UnityViewPagedChildren( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Checks the nesting of categories and then calls ViewPagedChildren
#----------------------------------------------------------------------------------------
# §input        $Servlet | current servlet (contains form data) | object
# §return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub UnityViewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;

    if($Object->countCategoryNesting >= MAX_NESTING_OF_CATEGORIES) {
        $Servlet->vars('TooManyLevelsOfCategories', {
            MaxValue    => MAX_NESTING_OF_CATEGORIES
        });
    }

    return $self->ViewPagedChildren($Servlet);
}

#========================================================================================
# §function     UploadPageEditorImage
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hVars = $Handler->UploadPageEditorImage( $Servlet );
#----------------------------------------------------------------------------------------
# §description  UI handler for uploads in the page preview. Generates both image variations
#               for list view and detail view.
#----------------------------------------------------------------------------------------
# §input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub UploadPageEditorImage {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::UploadFilesToMediaGallery($Servlet);

    my $Object = $Servlet->object;
    my $Site = $Object->getSite;

    my $hResponseContent = $Servlet->vars('ResponseContent');
    my $WebPath = $Site->get('WebPath');
    my $PublicPath = $Site->get('PublicPath');
    my $ObjectPublicPath = $Object->get('PublicPath');

    my $File = $hResponseContent->{'uploadedFile'};
    $File =~ s/^\Q$WebPath\E/$PublicPath/;

    my $Form = $Servlet->form;

    eval{
        TestImage($File);
    };
    if(ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'Form'  => 'UploadImages',
            'Name'  => $Error->code,
        });
    }

    my $MediumWidth =  $Form->value('Width') || CATEGORY_IMAGES_SIZES->{'ImageMedium'}->[0];
    my $MediumHeight = $Form->value('Height') || CATEGORY_IMAGES_SIZES->{'ImageMedium'}->[1];
    my $SmallWidth =  CATEGORY_IMAGES_SIZES->{'ImageSmall'}->[0];
    my $SmallHeight = CATEGORY_IMAGES_SIZES->{'ImageSmall'}->[1];
    my $Quality = $Form->value('Quality');

    # small image target file
    my ($TargetSmallPath, $TargetSmallFilename, $TargetSmallExt) = fsplit($File);
    my $TargetSmall = $TargetSmallPath . '/' . $TargetSmallFilename . '_list' . $TargetSmallExt;

    # scale image
    TestImageFormat( 'File', $File );
    eval {
        # read image only once
        my $OrgImage = Image::Magick->new;
        my $ImageError = $OrgImage->Read($File);
        Error('IMAGE_MAGICK', {'Function'=>'Read', 'File'=>$File, 'Message'=>$ImageError}) if $ImageError;

        # get image width and height
        my $OrgImageHeight = $OrgImage->Get('height');
        my $OrgImageWidth  = $OrgImage->Get('width');

        # possible image magick cant read width/height
        if (not defined $OrgImageWidth or not defined $OrgImageHeight) {
            my $ImageInfo = ImageInfo($File);
            if (defined($ImageInfo->{'ImageHeight'}) && defined($ImageInfo->{'ImageWidth'})) {
                # try to determine image dimensions via exif
                $OrgImageHeight = $ImageInfo->{'ImageHeight'};
                $OrgImageWidth  = $ImageInfo->{'ImageWidth'};
            } else {
                Error('UNKNOWN_IMAGE_SIZE', {'File' => $File, 'Width' => $OrgImageWidth, 'Height' => $OrgImageHeight });
            }
        }

        # fix clr space, meta data, etc.
        CorrectImage($File, $OrgImage);

        # create memcpy
        my $WorkImage = $OrgImage->Clone();

        # write page preview image for list view (ImageSmall)
        ResizeImageUsingImageRef(
            $WorkImage,
            $File,
            $TargetSmall,
            $OrgImageWidth,
            $OrgImageHeight,
            $SmallWidth,
            $SmallHeight,
            $Quality
        );

        # create memcpy
        $WorkImage = $OrgImage->Clone();

        # write page preview image for detail view (ImageMedium)
        ResizeImageUsingImageRef(
            $WorkImage,
            $File,
            $File,
            $OrgImageWidth,
            $OrgImageHeight,
            $MediumWidth,
            $MediumHeight,
            $Quality
        );

        # free mem
        undef $WorkImage;
        undef $OrgImage;
    };
    if (ExistsError()) {
        DeleteFile($TargetSmall);
        DeleteFile($File);
        my $Error = GetError();
        if($Error->code =~ /^IMAGE_MAGIC/) {
            $Form->executeFormError({
                'Reason'=> 'IMAGE_MAGICK',
                'Form'  => 'UploadImages',
                'Name'  => 'NewFile',
            });
        }
        Error( $Error );
    }

    # remove current images
    my $CurrentImageSmall = $Object->get('ImageSmall');
    if($CurrentImageSmall) {
        $CurrentImageSmall = $ObjectPublicPath.'/'.$CurrentImageSmall;
        DeleteFile($CurrentImageSmall) unless $CurrentImageSmall eq $TargetSmall;
    }
    my $CurrentImageMedium = $Object->get('ImageMedium');
    if($CurrentImageMedium) {
        $CurrentImageMedium = $ObjectPublicPath.'/'.$CurrentImageMedium;
        DeleteFile($CurrentImageMedium) unless $CurrentImageMedium eq $File;
    }

    # Save attributes
    $Object->set({
        'ImageSmall' => RelativePath($ObjectPublicPath, $TargetSmall),
        'ImageMedium' => RelativePath($ObjectPublicPath, $File)
    });

    # both changed files
    my @Files = ( $File, $TargetSmall );

    # get image info for each image in list
    my $hJsonImageData = GetJSONImagesInfo($ObjectPublicPath, \@Files);
    $hResponseContent->{$_} = $hJsonImageData->{$_} foreach keys %$hJsonImageData;

    # update servlet var ResponseContent
    $Servlet->vars('ResponseContent', $hResponseContent);

    return;
}

1;
