#========================================================================================
# package      DE_EPAGES::Database::API::BackupMySQL
# description  Functions in this modules are used with backup/restore MySQL databases
#========================================================================================
package DE_EPAGES::Database::API::BackupMySQL;

use base qw( Exporter );
use strict;

our @EXPORT_OK  = qw (
    BackupBinLogs
    GetBinLogFiles
    GetBinLogParams
    DeleteOldLogs
    RestoreBinLogs
    IgnoreDumpsNewerThan
    MyCnfDefault
    GetStoreConf
    GetMySQLConf
    SetMySQLConf
    MySQLShowVariables
    GetMySQLSettings
    MySQLConnectionDefault
    CheckMySQLSettings
    CheckLogFullArguments
    GetDatabaseSize
);

use Config;
use Carp;
use Getopt::Long;
use File::Spec;
use File::Basename  qw ( basename dirname );
use Sys::Hostname;
use Config::Extend::MySQL;
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::File qw( GetFileContentLines CopyFile CorrectPathSlashes
                                   DeleteFile IsFileNewer ExistsFile GetFileNames
                                   ExistsDirectory AbsolutePath fsplit
                                   CreateDirectory SetAccessRights
                                   WindowsPath WriteFile );
use DE_EPAGES::Core::API::PerlTools qw( UsePackage GetIPAddress unique );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::Core::API::TestParameter qw( TestString TestInteger );
use DE_EPAGES::Core::API::Program::Tools qw( FindProgramFile );
use DE_EPAGES::Core::API::Constants;
use DE_EPAGES::Database::API::Config qw( DatabaseConnectionData DatabaseBackupData
                                         DatabaseConfigFileName );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore );

BEGIN {
    UsePackage( 'Win32' ) if $Config{'osname'} eq 'MSWin32';
}

#========================================================================================
# function     BackupBinLogs
# state        public
#----------------------------------------------------------------------------------------
# syntax       BackupBinLogs($hSettings, $Verbose);
# example      BackupBinLogs(GetMySQLSettings($Store), $Verbose);
#----------------------------------------------------------------------------------------
# description  Copies binary log files to backup location; this location is
#               created in $hSettings->{DumpDir}/binlog/$hSettings->{BinLogSubDir}
#----------------------------------------------------------------------------------------
# input        $hSettings | output of GetMySQLSettings(), see below | ref hash
# input        $Verbose   | print messages to stdout | string
# input        $Flush     | 0 - do not flush binlogs, copy only (default: 1) | boolean
#========================================================================================
sub BackupBinLogs ($;$$) {
    my ($hSettings, $Verbose, $Flush) = @_;
    $Flush = 1 unless defined $Flush;

    my $FlushCommand = "mysqladmin flush-logs $hSettings->{BackupOptions}";
    SystemCommand( $FlushCommand , { 'Print' => $Verbose } ) if $Flush;

    # Find all binlog files in mysqlbinlog.index.
    my $aBinLogFiles = GetBinLogFiles($hSettings);

    # Leave out the last file, because it was generated right after the flush-logs command
    # above and may be still open by other writing processes.
    pop @$aBinLogFiles if $Flush;

    # Copy each binlog file to backup dir if it was not copied before.
    my $LogDirName = File::Spec->catfile($hSettings->{DumpDir}, 'binlog');
    $LogDirName = File::Spec->catfile($LogDirName, $hSettings->{BinLogSubDir})
        if $hSettings->{BinLogSubDir};

    # use explict access rights because $LogDirName might be outside of eproot/
    CreateDirectory($LogDirName, { User => 'ep_appl', Group => 'epages',
                                   Mode => GROUP_RX } );

    foreach my $SrcFile (@$aBinLogFiles) {
        my (undef, $FileName, $Extension) = fsplit($SrcFile);
        my $DestFile = File::Spec->catfile( $LogDirName, $FileName.$Extension );

        # Don't copy existing binlog files again
        next if ExistsFile("$DestFile.bz2") or ExistsFile("$DestFile.gz");
        next if ExistsFile($DestFile) and IsFileNewer($SrcFile, $DestFile);

        print "Copying binlog file $SrcFile to $DestFile\n" if $Verbose;
        CopyFile( $SrcFile, $DestFile, { stat => 'ignore' });
        SetAccessRights($DestFile, { User => 'ep_appl', Group => 'epages',
                                     Mode => GROUP_R } );
    }
}

#========================================================================================
# function     GetBinLogFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aBinLogFiles = GetBinLogFiles($hSettings);
# example      my $aBinLogFiles = GetBinLogFiles(GetMySQLSettings($Store));
#----------------------------------------------------------------------------------------
# description  Returns all binlog files contained in the binlog index file
#----------------------------------------------------------------------------------------
# input        $hSettings | output of GetMySQLSettings(), see below | ref hash
# return       $aBinLogFiles | array whith absolute filenames | ref.array
#========================================================================================
sub GetBinLogFiles {
    my ($hSettings) = @_;

    # Get absolute pathes for log_bin and log_bin_index.
    my ($LogBin, $LogBinIndex) = GetBinLogParams( $hSettings );

    # Return if no log_bin is found.
    return [] unless $LogBin;

    # Get binlog file names. These may be absolute or relative.
    my $aBinLogFiles = GetFileContentLines( $LogBinIndex );

    # Convert relative file names to absolute ones
    my $BinLogDir = CorrectPathSlashes(dirname($LogBin));

    return [
        map {
            chomp($_);
            AbsolutePath($_, $BinLogDir);
        }
        @$aBinLogFiles
    ];
}

#========================================================================================
# function     GetBinLogParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($LogBin, $LogBinIndex) = GetBinLogParams( $hSettings );
# example      my ($LogBin, $LogBinIndex) = GetBinLogParams(GetMySQLSettings($Store));
#----------------------------------------------------------------------------------------
# description  Returns the absolute pathes for log_bin and log_bin_index
#----------------------------------------------------------------------------------------
# input        $hSettings   | output of GetMySQLSettings(), see below | ref hash
# return       $LogBin      | absolute path to log_bin (if any, else undef) | string
# return       $LogBinIndex | absolute path to log_bin_index (if any, else undef) | string
#========================================================================================
sub GetBinLogParams {
    my ($hSettings) = @_;

    # To keep track of which binary log files have been used, mysqld also creates a binary log index
    # file that contains the names of all used binary log files. By default, this has the same
    # basename as the binary log file, with the extension '.index'. You can change the name of the
    # binary log index file with the --log-bin-index[=file_name]  option.

    my $LogBin = $hSettings->{log_bin};
    my $LogBinIndex = $hSettings->{log_bin_index};
    my $DataDir = $hSettings->{datadir};

    # Set $LogBin to default value unless defined.
    # (see http://dev.mysql.com/doc/refman/5.5/en/binary-log.html)
    if ( not $LogBin and $DataDir ) {
        my $FileName = $hSettings->{hostname};
        $FileName = hostname() unless $FileName;
        $FileName = (fsplit $hSettings->{pid_file})[1] if $hSettings->{pid_file};
        $LogBin = AbsolutePath("$FileName-bin.log", $DataDir);
    } elsif ($LogBin) {
        my $IsAbsolute = File::Spec->file_name_is_absolute($LogBin);
        if (not $IsAbsolute) {
            $LogBin = $DataDir ? AbsolutePath($LogBin, $DataDir) : undef;
        }
    }

    # If index file entry does not exists, we build the name by the binary log files names.
    if ( not $LogBinIndex and $LogBin) {
        my $BaseName = basename($LogBin);
        # If you supply an extension in the log name (for example, --log-bin=base_name.extension),
        # the extension is silently removed and ignored by mysql.
        $BaseName =~ s/\.log$//;
        $LogBinIndex = AbsolutePath($BaseName.'.index', dirname($LogBin));
    } elsif ($LogBinIndex) {
        my $IsAbsolute = File::Spec->file_name_is_absolute($LogBinIndex);
        if (not $IsAbsolute) {
            my $DirName = $LogBin ? dirname($LogBin) : $DataDir;
            $LogBinIndex = $DirName ? AbsolutePath($LogBinIndex, $DirName) : undef;
        }
    }

    return ($LogBin, $LogBinIndex);
}

#========================================================================================
# function     DeleteOldLogs
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteOldLogs($DumpDir, $BinLogSubDir, $Days, $Verbose);
# example      DeleteOldLogs('/tmp', undef, 14, 1);
#----------------------------------------------------------------------------------------
# description  Deletes binary logfiles which are older than $Days days.
#----------------------------------------------------------------------------------------
# input        $DumpDir | destination where logfiles were backed up to | string
# input        $BinLogSubDir | backup to $DumpDir/binlog/$BinLogSubDir | string
# input        $Days  | number of days to keep old binlog files | integer
# input        $Verbose | print messages to stdout | string
#========================================================================================
sub DeleteOldLogs {
    my ( $DumpDir, $BinLogSubDir, $Days, $Verbose ) = @_;

    # Get the binlog files.
    my $LogDir = File::Spec->catfile( $DumpDir, 'binlog' );
    $LogDir = File::Spec->catfile($LogDir, $BinLogSubDir) if $BinLogSubDir;
    return unless ExistsDirectory($LogDir);

    # Get files older than $Days days
    my @OldFiles = grep { -M $_ >= $Days } @{ GetFileNames( $LogDir, { 'recursive' => 0 } ) };
    foreach my $DumpFile (@OldFiles) {
        print "Deleting $DumpFile\n" if $Verbose;
        DeleteFile( $DumpFile );
    }

    if ( $Verbose && 0 == scalar @OldFiles ) { print "No old binlog files found.\n" }
    return;
}

#========================================================================================
# function     RestoreBinLogs
# state        public
#----------------------------------------------------------------------------------------
# syntax       RestoreBinLogs($StartLogFile, $StartPosition, $EndLogFile, $EndPosition,
#                              $hSettings, $Verbose);
#----------------------------------------------------------------------------------------
# description  Restores backed up log files at start position from the directory
#               $hSettings->{DumpDir}/binlog/.
#               Does nothing if dump directory does not exist.
#----------------------------------------------------------------------------------------
# input        $StartLogFile | name of first log file to apply| string
# input        $StartPosition | number of first log position to apply| integer
# input        $EndLogFile | name of last log file to apply (optional) | string
# input        $EndPosition | number of last log position to apply (optional) | integer
# input        $hSettings | output of GetStoreConf(), see below | ref hash
# input        $Verbose | print messages to stdout | string
#========================================================================================
sub RestoreBinLogs {
    my ($StartLogFile, $StartPosition, $EndLogFile, $EndPosition, $hSettings, $Verbose) = @_;

    TestString('StartLogFile', $StartLogFile );
    TestInteger('StartPosition', $StartPosition );

    print "Restoring logs at file $StartLogFile, position $StartPosition\n" if $Verbose;

    # Get the binlog files.
    my $LogDir = File::Spec->catfile( $hSettings->{DumpDir}, 'binlog' );
    $LogDir = File::Spec->catfile($LogDir, $hSettings->{BinLogSubDir})
        if $hSettings->{BinLogSubDir};
    return unless ExistsDirectory($LogDir);

    my ($StartLogFileNumber, $EndLogFileNumber);
    if ($StartLogFile =~ /(\d+)$/) {
        $StartLogFileNumber = $1;
    } else {
        croak "Can't find log file number in file name '$StartLogFile'";
    }
    $EndLogFileNumber = 0;
    if( defined $EndLogFile ) {
        if ($EndLogFile =~ /(\d+)$/) {
            $EndLogFileNumber = $1;
        } else {
            croak "Can't find log file number in file name '$EndLogFile'";
        }
    }

    # Get log files files newer than last full dump.
    my @BinLogFiles;
    foreach my $File (@{GetFileNames($LogDir)}) {
        if ($File =~ /(\d+)$/) {
            next if $1 < $StartLogFileNumber;
            next if $1 > $EndLogFileNumber and defined $EndLogFile;
        }
        push @BinLogFiles, $File;
    }
    @BinLogFiles = sort @BinLogFiles;

    my $DBName = $hSettings->{Database};
    my $DBArgs = $hSettings->{BackupOptions};
    # Load transactions.
    my $BinLogCommand = "mysqlbinlog $DBArgs -d $DBName";
    $BinLogCommand .=  ' --read-from-remote-server' unless $hSettings->{LocalDB};

    my $RestoreCommand = "mysql $DBArgs -D $DBName";

    if( scalar @BinLogFiles > 0 ) {
        $BinLogCommand .= " --start-position=$StartPosition";
        $BinLogCommand .= " --stop-position=$EndPosition" if $EndPosition;
        $BinLogCommand .= " $_" foreach @BinLogFiles;
        SystemCommand( $BinLogCommand . ' | '. $RestoreCommand, { 'Print' => $Verbose } );
    }
    else {
        print "No logfiles found.\n" if $Verbose;
    }

    return;
}

#========================================================================================
# function     IgnoreDumpsNewerThan
# state        public
#----------------------------------------------------------------------------------------
# syntax       IgnoreDumpsNewerThan($DumpDate, $hSettings);
# example      IgnoreDumpsNewerThan($DumpDate, GetStoreConf($Store));
#----------------------------------------------------------------------------------------
# description  Renames files newer than $DumpDate in $hSettings->{DumpDir}
#----------------------------------------------------------------------------------------
# input        $DumpDate | date with format YYYYMMDDHHMMSS | string
# input        $hSettings | output of GetStoreConf(), see below | ref hash
#========================================================================================
sub IgnoreDumpsNewerThan {
    my ($DumpDate, $hSettings) = @_;

    my $DumpDir = $hSettings->{DumpDir};
    my $DBName  = $hSettings->{Database};

    return unless $DumpDir =~ /\S/;

    if ($DumpDate =~ /^[0-9]+$/) {
        # move new dumps
        foreach my $DumpFile ( glob "$DumpDir/${DBName}_*.dmp" ) {
            my $TimeStamp = $DumpFile =~ /.*_([0-9]+)\.dmp$/x ? $1 : '';
            while (length($TimeStamp) < length($DumpDate)) {
                $TimeStamp .= '0';
            }
            if ($TimeStamp > $DumpDate) {
                rename $DumpFile, "$DumpFile.new" or die "rename($DumpFile,$DumpFile.new): $!\n";
            }
        }
    }
    return;
}

#========================================================================================
# function     MyCnfDefault
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ConfigFileName = MyCnfDefault($BaseDir);
#----------------------------------------------------------------------------------------
# description  Returns the best guess for MySQL config file name (file might not exist)
#               depending on $Basedir (from MySQLShowVariables()->{basedir}, see below)
#----------------------------------------------------------------------------------------
# input        $BaseDir        | MySQL base dir | string
# return       $ConfigFileName | MySQL config file name | string
#========================================================================================
sub MyCnfDefault {
    my ($BaseDir) = @_;

    if ($Config{osname} ne 'MSWin32') {
        foreach (('/etc/my.cnf', '/etc/mysql/my.cnf', '/etc/my.cnf.d/server.cnf')) {
            return $_ if -f $_;
        }
        return "$BaseDir/my.cnf" if defined $BaseDir and -f "$BaseDir/my.cnf";
        return '/etc/my.cnf';
    }

    # ------------------------------------------------------------------------
    # windows tests
    # ------------------------------------------------------------------------

    # 1st try: find my.ini in $BaseDir:
    if ($BaseDir) {
        $BaseDir =~ s/\\+$//;
        return $BaseDir. '/my.ini' if -f "$BaseDir/my.ini";
    }

    # 2nd try: find my.ini in $ENV{PROGRAMFILES}:
    my $MyIni = FindProgramFile('MySQL/MySQL Server*/my.ini');
    return $MyIni if $MyIni;

    # 3rd try: find my.ini in $ENV{PATH}:
    foreach my $Dir (split(/;/, $ENV{PATH})) {
        return WindowsPath(dirname($Dir). '/my.ini') if -f dirname($Dir). '/my.ini';
    }

    # otherwise: my.ini not found, but should be here:
    my $DefaultVersion = 'MySQL Server 5.5';
    return WindowsPath("C:/Program Files/MySQL/$DefaultVersion/my.ini");
}

#========================================================================================
# function     GetStoreConf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSettings = GetStoreConf($Store, $BackupStore);
#----------------------------------------------------------------------------------------
# description  Returns hash of ConnectionData retrieved from
#               DatabaseConnectionData($Store) and DatabaseBackupData($Store,$BackupStore).
#----------------------------------------------------------------------------------------
# input        $Store       | name of the store which should be used (default: Backup) | string
# input        $BackupStore | database connection name (optional,
#                              default: DatabaseBackupData($Store)->{Backup}) | string
# return       $hSettings   | keys as in DatabaseConnectionData($Store) plus keys as in
#                              DatabaseBackupData($Store,$BackupStore) prefixed with Backup_
#                              plus additional keys:
#                   <ul>
#                       <li>StoreName          - $Store
#                       <li>LocalDB            - 1, if datasource is socket or hostname is myself
#                       <li>FileSocket         - mysql file socket (from datasource)
#                       <li>TCPSocket          - mysql TCP socket (from datasource)
#                       <li>SocketOptions      - mysql options -S, -h, -P (from datasource)
#                       <li>StoreOptions       - SocketOptions -ulogin -ppasswd
#                       <li>BackupOptions      - SocketOptions -uBackup_login -pBackup_passwd
#                   </ul>
#               | ref.hash
#========================================================================================
sub GetStoreConf ($;$) {
    my ($Store, $BackupStore) = @_;
    $Store = 'Backup' unless $Store;

    my $hSettings = DatabaseConnectionData($Store);

    $hSettings->{StoreName} = $Store;

    $hSettings->{BinLogSubDir} =~ s,^/*(.+)/*$,$1,;
    $hSettings->{DumpDir} =~ s/_(.+?)_/$ENV{$1}/g;
    $hSettings->{DumpDir} =~ s/\$ENV{(.+?)}/$ENV{$1}/g;

    my $hBackupSettings = DatabaseBackupData($Store, $BackupStore);
    $hSettings->{"Backup_$_"} = $hBackupSettings->{$_} foreach keys %$hBackupSettings;

    return $hSettings unless IsMySQLDB($hSettings);

    # get additional MySQL keys: SocketOptions LocalDB StoreOptions BackupOptions

    my $HostName;
    if ($hSettings->{datasource} =~ /mysql_socket=(.+?)(;|\s*$)/x) {
        $hSettings->{FileSocket} = "$1";
        $hSettings->{SocketOptions} = " -S$1";
    } elsif ($hSettings->{datasource} =~ /host(name)?=(.+?)(;|\s*$)/x) {
        $HostName = $2;
        $hSettings->{TCPSocket} = $HostName;
        $hSettings->{SocketOptions} = " -h$HostName";
        if ($hSettings->{datasource} =~ /port=(.+?)(;|\s*$)/x)  {
            $hSettings->{TCPSocket} = "$HostName;$1";
            $hSettings->{SocketOptions} .= " -P$1";
        }
    } elsif ($hSettings->{datasource} =~ /port=(.+?)(;|\s*$)/x) {
        $hSettings->{TCPSocket} = ";$1";
        $hSettings->{SocketOptions} = " -P$1";
    }
    $hSettings->{LocalDB} = 1 if GetIPAddress($HostName) eq GetIPAddress();

    $hSettings->{StoreOptions} = $hSettings->{SocketOptions};
    $hSettings->{StoreOptions} .= " -u$hSettings->{login}" if $hSettings->{login};
    $hSettings->{StoreOptions} .= " -p$hSettings->{passwd}" if $hSettings->{passwd};

    if ( scalar keys %$hBackupSettings ) {
        $hSettings->{BackupOptions} = $hSettings->{SocketOptions};
        $hSettings->{BackupOptions} .= " -u$hSettings->{Backup_login}" if $hSettings->{Backup_login};
        $hSettings->{BackupOptions} .= " -p$hSettings->{Backup_passwd}" if $hSettings->{Backup_passwd};
    }

    foreach (keys %$hSettings) {
        $hSettings->{$_} = undef unless $hSettings->{$_};
    }

    return $hSettings;
}

sub IsMySQLDB {
    my ($hSettings) = @_;

    if ($hSettings->{datasource} =~ /^dbi:mysql/i or $hSettings->{datasource} =~ /^dbi:galera/i) {
        return 1;
    }
    else {
        return 0;
    }
}

#========================================================================================
# function     GetMySQLConf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSettings = GetMySQLConf($MySQLConfigFile);
#----------------------------------------------------------------------------------------
# description  Returns hash of MySQL settings from $MySQLConfigFile.
#               In $hSettings returned parameters can just exist (no-value parameters)
#               or be defined by a value. Thatswhy, check both exists($hSettings->{})
#               and/or defined($hSettings->{}).
#               All hyphens (-) in keys in $hSettings are translated to underscores(_).
#               You won't find $hSettings->{'log-bin'}, but $hSettings->{log_bin}.
#----------------------------------------------------------------------------------------
# input        $MySQLConfigFile | location of my.cnf/my.ini | string
# return       $hSettings | hash with server system variables as defined in
#                            http://dev.mysql.com/doc/refman/5.5/en/server-system-variables.html
#                          | ref hash
#========================================================================================
sub GetMySQLConf {
    my ($MySQLConfigFile) = @_;
    my %Settings;

    return {} unless $MySQLConfigFile;
    return {} unless -r $MySQLConfigFile;

    my $IniConfig = Config::Extend::MySQL->new({ from => $MySQLConfigFile,
                                                 using => 'Config::IniFiles' });
    foreach my $Section (qw(server mysqld mysqld_safe)) {
        foreach my $Parameter ($IniConfig->Parameters($Section)) {
            my $Value = $IniConfig->val($Section, $Parameter);
            $Value = undef if $Value eq 'yes'; # value is defined but empty
            $Value = $2 if $Value =~ /^(['"])(.*)\1$/x; #' unquote value
            #  all '-' in keys in $hSettings are translated to '_':
            $Parameter =~ s,-,_,g;
            $Settings{$Parameter} = $Value unless defined $Settings{$Parameter};
        }
    }

    foreach (keys %Settings) {
        $Settings{$_} = undef unless $Settings{$_};
    }

    return \%Settings;
}

#========================================================================================
# function     SetMySQLConf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Result = SetMySQLConf($Section, $Key, $Value, $hFlags);
#----------------------------------------------------------------------------------------
# description  Sets in /etc/my.cnf (from MyCnfDefault() or $hFlags) in [$Section]
#               $Key=$Value. Returns success (1, if set) or failure (0, file not writeable).
#----------------------------------------------------------------------------------------
# input        $Section | section of /etc/my.cnf (req.) | string
# input        $Key     | key in section (req.) | string
# input        $Value   | value of key (req.) | string
# input        $hFlags  | flags
#                   <ul>
#                       <li>my_cnf - use this dedicated my.cnf
#                       <li>store  - try to get my.cnf from Database.d/store.conf->MySQLConfigFile
#                   </ul>
#               | ref.hash
# return       $Result  | success (1) or failure (0) | boolean
#========================================================================================
sub SetMySQLConf ($$$;$) {
    my ($Section, $Key, $Value, $hFlags) = @_;

    my $MyCnf = $hFlags->{my_cnf};
    if (not defined $MyCnf and defined $hFlags->{store}) {
        $MyCnf = GetStoreConf($hFlags->{store})->{MySQLConfigFile};
    }

    $MyCnf //= MyCnfDefault();

    return 0 unless -w $MyCnf;

    # unluckily we can't use Config::IniFiles for my.cnf
    # because it contains extended ini syntax
    my ($Content, $SectionFound, $KeyFound);
    foreach my $Line ( @{GetFileContentLines($MyCnf)} ) {
        if (not defined $SectionFound and $Line =~ /^\s*\[$Section\]/) {
            $SectionFound = 1;
        } elsif ($SectionFound and $Line =~ /^\s*\[.+\]/) {
            $SectionFound = 0;
            $Content .= "$Key=$Value\n" unless $KeyFound;
            $KeyFound = 1;
        } elsif ($SectionFound and $Line =~ s,^(\s*$Key\s*=\s*).*,$1$Value,) {
            $KeyFound = 1;
        }
        $Content .= $Line;
    }
    # add $Section/$Key/$Value to the end of my.cnf unless already set
    if (not $KeyFound) {
        $Content .= "[$Section]\n" if not defined $SectionFound;
        $Content .= "$Key=$Value\n";
    }
    WriteFile($MyCnf, \$Content);

    return 1;
}

#========================================================================================
# function     GetDatabaseSize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SizeInBytes = GetDatabaseSize($StoreName);
#----------------------------------------------------------------------------------------
# description  Returns the size in bytes (Data_length + Index_length) of the database
#               of $StoreName.
#----------------------------------------------------------------------------------------
# input        $StoreName | name of the store which should be used (default: Backup) | string
# return       $SizeInBytes | database size in bytes | integer
#========================================================================================
sub GetDatabaseSize ($) {
    my ($StoreName) = @_;

    my $SizeInBytes = 0;
    RunOnStore( 'Store' => $StoreName, 'Sub' => sub {
        my $aaRows = GetCurrentDBHandle()->execute('SHOW TABLE STATUS');
        foreach (@$aaRows) {
            $SizeInBytes += $_->[6] + $_->[8]; # Data_length + Index_length
        }
                });
    return $SizeInBytes;
}

#========================================================================================
# function     MySQLShowVariables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSettings = MySQLShowVariables($StoreName);
#----------------------------------------------------------------------------------------
# description  Returns hash of MySQL settings for $StoreName from 'SHOW VARIABLES'.
#               In $Settings returned parameters can just exist (no-value parameters)
#               or be defined by a value. Thatswhy check both exists($hSettings->{})
#               and/or defined($hSettings->{}).
#----------------------------------------------------------------------------------------
# input        $StoreName | name of the store which should be used (default: Backup) | string
# return       $hSettings | hash with SHOW VARIABLES as defined in
#                            http://dev.mysql.com/doc/refman/5.5/en/show-variables.html
#                            additional key 'connected' shows if connection works | ref hash
#========================================================================================
sub MySQLShowVariables {
    my ($StoreName) = @_;
    $StoreName = 'Backup' unless $StoreName;

    my %Settings;
    eval { RunOnStore( 'Store' => $StoreName, 'Sub' => sub {
        my $aaRows = GetCurrentDBHandle()->execute('SHOW VARIABLES');
        foreach (@$aaRows) {
            my ($Parameter, $Value) = ($_->[0], $_->[1]);
            next if $Value =~ /^(FALSE|OFF)$/; # value is not defined
            $Value = undef if $Value =~ /^(TRUE|ON)$/; # value is defined but empty
            $Settings{$Parameter} = $Value;
        }
                       })
    };
    $Settings{connected} = ExistsError() ? 0 : 1;

    foreach (keys %Settings) {
        $Settings{$_} = undef unless $Settings{$_};
    }

    return \%Settings;
}

#========================================================================================
# function     GetMySQLSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSettings = GetMySQLSettings($Store, $BackupStore);
#----------------------------------------------------------------------------------------
# description  Returns hash of combined output of MySQLShowVariables(), GetStoreConf()
#               and GetMySQLConf(), see above.
#----------------------------------------------------------------------------------------
# input        $Store       | name of the store which should be used (default: Backup) | string
# input        $BackupStore | database connection name (optional,
#                              default: DatabaseBackupData($Store)->{Backup}) | string
# input        $MySQLConfigFile | location of dedicated my.cnf/my.ini (optional) | string
# return       $hSettings   | hash with the collected data | hash
#========================================================================================
sub GetMySQLSettings ($;$$) {
    my ($Store, $BackupStore, $MySQLConfigFile) = @_;
    $Store = 'Backup' unless $Store;

    my $hSettings = { %{MySQLShowVariables($Store)}, %{GetStoreConf($Store, $BackupStore)} };

    $MySQLConfigFile = $hSettings->{MySQLConfigFile} unless $MySQLConfigFile;
    $MySQLConfigFile = MyCnfDefault($hSettings->{basedir}) unless $MySQLConfigFile;
    $hSettings->{MySQLConfigFile} = -r $MySQLConfigFile ? $MySQLConfigFile : undef;

    my $hMySQLConf = GetMySQLConf($MySQLConfigFile);

    foreach (keys %$hMySQLConf) {
        # we rely more on MySQLShowVariables() than on GetMySQLConf():
        # overwrite only if undefined or add if not existing
        $hSettings->{$_} = $hMySQLConf->{$_} unless defined $hSettings->{$_};
    }

    # set defaults for innodb directories
    if ($hSettings->{innodb_data_home_dir}) {
        $hSettings->{innodb_data_home_dir} =
            AbsolutePath($hSettings->{innodb_data_home_dir}, $hSettings->{datadir});
    } else {
        $hSettings->{innodb_data_home_dir} = $hSettings->{datadir};
    }
    if ($hSettings->{innodb_log_group_home_dir}) {
        $hSettings->{innodb_log_group_home_dir} =
            AbsolutePath($hSettings->{innodb_log_group_home_dir}, $hSettings->{datadir});
    } else {
        $hSettings->{innodb_log_group_home_dir} = $hSettings->{innodb_data_home_dir};
    }

    # set default for binlog backup
    $hSettings->{log_bin} = '' if exists $hSettings->{log_bin} && !$hSettings->{log_bin};

    return $hSettings;
}

#========================================================================================
# function     CheckMySQLSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ErrorMsg = CheckMySQLSettings($hSettings, [@Defined], [@Exists], $Store, $MySQLConfigFile);
# example      my $ErrorMsg = CheckMySQLSettings(GetMySQLSettings($Store));
#----------------------------------------------------------------------------------------
# description  Checks if $hSettings contains required keys plus @Defined and @Exists.
#----------------------------------------------------------------------------------------
# input        $hSettings | output of GetMySQLSettings(), see above | ref hash
# input        $Defined   | keys that must be defined | ref array
# input        $Exists    | keys that must exist | ref array
# input        $Store     | store name | string
# input        $MySQLConfigFile | name of MySQL config file | string
# return       $ErrorMsg  | error msg contains missing settings | string
#========================================================================================
sub CheckMySQLSettings ($;$$$$) {
    my ($hSettings, $Defined, $Exists, $Store, $MySQLConfigFile) = @_;

    my @RequiredSettings = qw(StoreName DumpDir Database datasource);
    my @MissingKeys = grep { not defined $hSettings->{$_} } @RequiredSettings;

    my $ErrorMsg;

    if (scalar @MissingKeys) {
        $Store = '*' unless $Store;
        my $ConfigFile = DatabaseConfigFileName($Store);
        $ErrorMsg .= "Error: Missing settings in $ConfigFile:\n";
        $ErrorMsg .= "[Connection]\n";
        $ErrorMsg .= "$_=$hSettings->{$_}\n" foreach @MissingKeys;
    }

    $Exists = [] unless defined $Exists;
    $Defined = [] unless defined $Defined;
    @MissingKeys = grep { not defined $hSettings->{$_} } unique sort @$Defined;
    push @MissingKeys, grep { not exists $hSettings->{$_} } unique sort @$Exists;

    if (scalar @MissingKeys) {
        $MySQLConfigFile = $hSettings->{MySQLConfigFile} unless $MySQLConfigFile;
        $ErrorMsg .= "Error: Missing settings in $MySQLConfigFile:\n";
        $ErrorMsg .= "[mysqld]\n";
        $ErrorMsg .= "$_=$hSettings->{$_}\n" foreach @MissingKeys;
    }

    return $ErrorMsg ? "\n$ErrorMsg\n" : '';
}

#========================================================================================
# function     MySQLConnectionDefault
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ENV{EPAGES_MYSQL_CONNECTION} = MySQLConnectionDefault();
#----------------------------------------------------------------------------------------
# description  Returns the best guess for environment variable EPAGES_MYSQL_CONNECTION.
#----------------------------------------------------------------------------------------
# return       $EPAGES_MYSQL_CONNECTION | environment variable value | string
#========================================================================================
sub MySQLConnectionDefault () {
    my $hSettings = MySQLShowVariables();
    $hSettings->{hostname} //= hostname();
    $hSettings->{hostname} = hostname() if $hSettings->{hostname} eq 'localhost';

    return $hSettings->{hostname} unless $hSettings->{port};
    return $hSettings->{hostname} if $hSettings->{port} eq '3306';
    return "$hSettings->{hostname};$hSettings->{port}";
}

#========================================================================================
# function     CheckLogFullArguments
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ErrorMsg = CheckLogFullArguments($Full,$Log,$DelLog,$MvOld);
#----------------------------------------------------------------------------------------
# description  Checks, if all necessary parameters are set for log/full backup.
#----------------------------------------------------------------------------------------
# input        $Full         | value of -full script parameter | string
# input        $Log          | value of -log script parameter | string
# input        $DelLog       | value of -dellog script parameter | string
# input        $MvOld        | value of -mvold script parameter  | string
# return       $ErrorMsg     | error msg contains wrong parameters | string
#========================================================================================
sub CheckLogFullArguments {
    my ($Full, $Log, $DelLog, $MvOld) = @_;

    return 'Error: Missing parameter -full or -log' unless $Full or $Log;
    return 'Error: Please choose either full or incremental backup.' if $Full and $Log;
    return 'Error: Please choose -dellog only when doing a full backup.' if $DelLog and $Log;
    return 'Error: Please choose -mvold only when doing a full backup.' if $MvOld and $Log;
    return undef;
}

1;
