#========================================================================================
# package      DE_EPAGES::ELogistics::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::ELogistics::API::Cartridge;
use base DE_EPAGES::Shipping::API::ShippingInstaller;

use DE_EPAGES::Object::API::Factory qw( LoadObject LoadObjectByPath ExistsObjectByPath ExistsClassByAlias);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle Transaction);
use DE_EPAGES::Core::API::PerlTools qw ( GetLoopFromDoubleArray );

use strict;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::ELogistics::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::ELogistics.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/ELogistics',
        'Version'            => '7.15.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    # exclusive Cartridge - check License Key before installing
    $self->testLicenseKey($self->cartridgePackage,'dfu9MrQLW5I3Pe7lOTNR');

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}
#========================================================================================
# function     patch_6_15_1
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patch_6_15_1;
#----------------------------------------------------------------------------------------
# description  EPG-27227 - Patch for EPG-26862: convert attributes from
#               ShippingFulfilmentType to ShippingFulfilmentTypeELogistics
# return       $newVersion | 6.15.2 | string
#========================================================================================
sub patch_6_15_1 {
  my $self = shift;

    #calling data changes from patch.xml file
    $self->logInfo('import new shippingFulfillmentTypeELogistics class');
    $self->importPatchDir('Patch_6_15_1', 'Patch');

    #moving ELogistics ShippingFulfillmentType object to ShippingFulfillmentTypeELogistics instance
    my @aELogisticsAttributes = ( qw (
         WebservicePickPostURL
         WebservicePickPostTestURL
         GoogleApiKeyPickPost
         PricingURL));

    $self->logInfo('move SFT ELogistics object from ShippingFulfillmentType to ShippingFulfillmentTypeELogistics class');

    unless (ExistsClassByAlias('ShippingFulfillmentTypeELogistics')) {
        $self->logInfo('ShippingFulfillmentTypeELogistics class doesn\'t exist. End patch');
        return '6.15.2';
    }

    my $dbi = $self->GetCurrentDBHandle();
    Transaction ( sub {
    if (ExistsObjectByPath('/ShippingFulfillmentTypes/ELogistics')) {
        my $ObjectID = LoadObjectByPath('/ShippingFulfillmentTypes/ELogistics')->id;
        my $NewClassID = $self->getClassID('ShippingFulfillmentTypeELogistics');
        $dbi->do(<<END_SQL, [$NewClassID, $ObjectID]);
UPDATE object
SET classid = ?
WHERE objectid = ?
END_SQL
        $self->logInfo('/ShippingFulfillmentTypes/ELogistics object moved to ShippingFulfillmentTypeELogistics class.');
    }
    else {
        $self->logInfo('/ShippingFulfillmentTypes/ELogistics object doesn\'t exist.');
    }

    #reassign class of attributes from ShippingFulfillmentType to ShippingFulfillmentTypeELogistics
    $self->logInfo('reassign attributes from ShippingFulfillmentType to ShippingFulfillmentTypeELogistics class');
    unless (ExistsObjectByPath('/Classes/ShippingFulfillmentType/Attributes')) {
        $self->logInfo('can\'t reassign nor delete because no ShippingFulfillmentType attributes exist. End patch.');
        return '6.15.2';
    }

    #get ELogistics attribute objects and set hash structure
    my %hResultSetNew = {};
    if (ExistsObjectByPath('/Classes/ShippingFulfillmentTypeELogistics/Attributes')) {
        my $NewParentID = LoadObjectByPath('/Classes/ShippingFulfillmentTypeELogistics/Attributes')->id;
        my $aaResultSet = $dbi->execute(<<END_SQL, [(@aELogisticsAttributes, $NewParentID)], ['String', 'Integer']);
SELECT obj.alias, obj.objectid
FROM object obj WHERE obj.alias IN (?, ?, ?, ?) AND obj.parentid = ?
END_SQL
        my $ahResultSetNew = GetLoopFromDoubleArray($aaResultSet, [qw ( alias objectid )]);
        %hResultSetNew = map { $_->{'alias'} => $_->{'objectid'} } @$ahResultSetNew;
    }

    my %hResultSetOld = {};
    if (ExistsObjectByPath('/Classes/ShippingFulfillmentType/Attributes')) {
        my $OldParentID = LoadObjectByPath('/Classes/ShippingFulfillmentType/Attributes')->id;
        my $aaResultSet = $dbi->execute(<<END_SQL, [(@aELogisticsAttributes, $OldParentID)], ['String', 'Integer']);
SELECT obj.alias, obj.objectid
FROM object obj WHERE obj.alias IN (?, ?, ?, ?) AND obj.parentid = ?
END_SQL
        my $ahResultSetOld = GetLoopFromDoubleArray($aaResultSet, [qw ( alias objectid )]);
        %hResultSetOld = map { $_->{'alias'} => $_->{'objectid'} } @$ahResultSetOld;
    }

    #move attribute values
    foreach my $Attribute (@aELogisticsAttributes) {
        next unless (defined($hResultSetNew{$Attribute}) && defined($hResultSetOld{$Attribute}));
        $self->logInfo('updating value for attribute ' . $Attribute . ' from ' . $hResultSetOld{$Attribute} . ' to ' . $hResultSetNew{$Attribute});
        $dbi->do(<<END_SQL, [$hResultSetNew{$Attribute}, $hResultSetOld{$Attribute}]);
UPDATE stringattribute
SET attributeid = ?
WHERE attributeid = ?
END_SQL
    }

    #delete former ELogistics attributes
    $self->logInfo('delete existing ELogistics attributes from ShippingFulfillmentType class');
    foreach my $Attribute (@aELogisticsAttributes) {
        next unless (defined($hResultSetOld{$Attribute}));
        my $Object = LoadObject($hResultSetOld{$Attribute});
        $self->logInfo('deleting old class attribute ' . $Attribute . '. ID: ' . $Object->id);
        $Object->delete();
    }

    $self->logInfo('patch finished');
    });
    return '6.15.2';
}

1;
