#========================================================================================
# package      DE_EPAGES::GoogleAnalytics::API::TLE::UniversalAnalyticsHandler
# description  Handle tracking with GoogleAnalytics.
# state        public
#========================================================================================
package DE_EPAGES::GoogleAnalytics::API::TLE::UniversalAnalyticsHandler;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( JoinObjectPath );

use constant UA_ECOMMERCE_VIEWACTIONS => [qw (
    View
    ViewMultiCheckoutCustomerOrder
    MobileSF-View
)];

use constant UA_BASKET_PAGEVIEW_SCHEME => ['/Basket', 'Basket'];
use constant UA_CHECKOUT_PAGEVIEW_SCHEME => ['/Checkout', 'Checkout'];

use constant UA_BASKET_PAGEVIEW_MAP => {
    'View'                    => UA_BASKET_PAGEVIEW_SCHEME,
    'MobileSF-View'           => UA_BASKET_PAGEVIEW_SCHEME,
    'ViewMultiCheckoutBasket' => UA_BASKET_PAGEVIEW_SCHEME,
    '*'                       => UA_CHECKOUT_PAGEVIEW_SCHEME
};


#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc($hParams);
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register($Processor);
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    GetLog->debug('UniversalAnalyticsHandler.register');
    $Processor->registerHandler('FunctionHandler', $self, 'IsUAEcommerceApplicable');
    $Processor->registerHandler('FunctionHandler', $self, 'UAAutoLinkerDomain');
    $Processor->registerHandler('FunctionHandler', $self, 'UAPageView');

    return;
}

#========================================================================================
# function     IsUAEcommerceApplicable
# state        public
#----------------------------------------------------------------------------------------
# syntax       #IsUAEcommerceApplicable(#Object, #ViewActionAlias)
#----------------------------------------------------------------------------------------
# example      #IF(#IsUAEcommerceApplicable(#Object, #INPUT.ViewAction)) ... #ENDIF
#----------------------------------------------------------------------------------------
# description  Determines if the combination of the current Object and ViewAction
#               should be used with the UniversalAnalytics' e-commerce functionality,
#               namely track a completed order.
#----------------------------------------------------------------------------------------
# input        $Processor
# input        $aParams |
#               <ul>
#                   <li>Object | current object | object</li>
#                   <li>ViewActionAlias | view action alias | string</li>
#               </ul> | ref.array
#----------------------------------------------------------------------------------------
# return       $IsApplicable | use UA's ecommerce functionality? | boolean
#========================================================================================
sub IsUAEcommerceApplicable {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $ViewActionAlias) = @$aParams;

    return 0 unless $Object->class->alias eq 'CustomerOrder';
    return (ElementOf($ViewActionAlias, UA_ECOMMERCE_VIEWACTIONS)) ? 1 : 0;
}

#========================================================================================
# function     UAAutoLinkerDomain
# state        public
#----------------------------------------------------------------------------------------
# syntax       #UAAutoLinkerDomain(#ShopObject, #Host)
#----------------------------------------------------------------------------------------
# example      #UAAutoLinkerDomain(#Shop.Object, #INPUT.HTTP_HOST)
#----------------------------------------------------------------------------------------
# description  If the shop's using two domains (shop domain and, for SSL, provider domain),
#               returns the domain which isn't used in the current request.
#               Google Analytic's auto linker will take care for the correct function.
#----------------------------------------------------------------------------------------
# input        $Processor
# input        $aParams |
#               <ul>
#                   <li>Shop | shop object | object</li>
#                   <li>Host | request host | string</li>
#               </ul> | ref.array
#----------------------------------------------------------------------------------------
# return       $AutoLinkerDomain | domain which should be auto-linked | string
#========================================================================================
sub UAAutoLinkerDomain {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $Host) = @$aParams;

    my $ShopDomainName = $Shop->get('DomainName');

    if (defined $ShopDomainName and not $Shop->get('HasSSLCertificate')) {
        if ($ShopDomainName eq $Host) {
            my $ProviderDomainName = $Shop->get('ProtocolAndServerSSL');
            $ProviderDomainName =~ m/https?:\/\/(.*?)(?:\/|$)/;
            return $1;
        }
        else {
            return $ShopDomainName;
        }
    }

    return $Host;
}

#========================================================================================
# function     UAPageView
# state        public
#----------------------------------------------------------------------------------------
# syntax       #UAPageView(#Object, #ViewActionAlias)
#----------------------------------------------------------------------------------------
# example      #LOCAL("PageView", #UAPageView(#Object, #INPUT.ViewAction)
#                   #PageView.Path
#                   #PageView.Title
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Provides page-view-specific info for the current object, with special
#               treatment for basket and checkout pages.
#----------------------------------------------------------------------------------------
# input        $Processor
# input        $aParams |
#               <ul>
#                   <li>Object | current object | object</li>
#                   <li>ViewActionAlias | current view action | string</li>
#               </ul> | ref.array
#----------------------------------------------------------------------------------------
# return       $hPageView | info hash for page view of the current object, keys:
#               <ul>
#                   <li>Path | easy-to-read object path | object</li>
#                   <li>Title | object title | string</li>
#               </ul> | ref.hash
#========================================================================================
sub UAPageView {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $ViewActionAlias) = @$aParams;

    my $result = eval {
        return $self->uaPageView($Object, $ViewActionAlias);
    };
    if (ExistsError()) {
        LogError(GetError());
        return {
            'Path'  => '',
            'Title' => ''
        };
    }
    else {
        return $result;
    }
}

sub uaPageView {
    my $self = shift;
    my ($Object, $ViewActionAlias) = @_;

    if ($Object->class->alias eq 'Basket') {
        my $aBasketPageView = defined UA_BASKET_PAGEVIEW_MAP->{$ViewActionAlias}
            ? UA_BASKET_PAGEVIEW_MAP->{$ViewActionAlias}
            : UA_BASKET_PAGEVIEW_MAP->{'*'};

        return {
            'Path'  => $Object->getSite->get('Path') . $aBasketPageView->[0],
            'Title' => $aBasketPageView->[1]
        };
    }
    elsif ($Object->class->alias eq 'User') {
        return {
            'Path'  => $self->_createAnonymizedPath($Object, 'ANONYMIZED_USER_PATH'),
            'Title' => 'ANONYMIZED_USER_NAME'
        };
    }
    elsif ($Object->class->alias eq 'ShoppingList') {
        return {
            'Path'  => $self->_createAnonymizedPath($Object, 'ANONYMIZED_SHOPPING_LIST_PATH'),
            'Title' => 'ANONYMIZED_SHOPPING_LIST_NAME'
        };
    }
    else {
        return {
            'Path'  => $Object->get('Path'),
            'Title' => $Object->get('NameOrAlias')
        };
    }
}

sub _createAnonymizedPath {
    my $self = shift;
    my ($Object, $PathExtension) = @_;

    my $Shop = $Object->getSite();
    my $ShopPath = $Shop->pathString;
    my @AnonymizedPathParts = split( /\//, $ShopPath );
    push( @AnonymizedPathParts, $PathExtension );

    return JoinObjectPath(\@AnonymizedPathParts);
}

1;
