use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( WriteFile );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath LoadObject LoadRootObject );

sub Main {
    local $| = 1;

    my ($Password, $StoreName, $Help, $OutFile);
    GetOptions(
        'help'          => \$Help,        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,   'file=s'        => \$OutFile,
    );

    usage() if $Help;
    usage("Missing parameter -storename" ) unless(defined $StoreName);
    usage("Missing parameter -file" ) unless(defined $OutFile);

    my $time1 = time;
    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            my $Content;
            $Content =  exportUsers();
            $Content .= exportProviderUsers();
            WriteFile ( $OutFile, \$Content,  {create_directory=>1} );
        },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub exportUsers {
    my ($BaseObject, $IdentLvl) = @_;

    $IdentLvl //= 0;
    $BaseObject = LoadRootObject() unless defined $BaseObject;
    my $Content = '';
    my $Users;
    if ($BaseObject->existsChild('Users')) {
        $Users = $BaseObject->child('Users')->children;
    } else {
        die sprintf("No child 'Users' on objectid %s (%s)",$BaseObject->id,$BaseObject->alias);
    }
    foreach my $child ( sort {$a->alias cmp $b->alias} @{$Users}) {
        my $Format = $IdentLvl*4;
        $Content .= sprintf "%${Format}sUser:%s\n",$IdentLvl>0 ? '- ' : '',$child->alias;
        my $Permissions = $child->get( 'Permissions' );
        foreach my $perm ( sort { LoadObject($a->{'MetaActionID'})->get(['Path'])->{'Path'} cmp LoadObject($b->{'MetaActionID'})->get(['Path'])->{'Path'} } @$Permissions ) {
            my $MetaAction = LoadObject($perm->{'MetaActionID'})->get(['Path'])->{'Path'};
            if ( $perm->{'TrusteeID'} eq $child->id ) {
                $Format = ($IdentLvl+1)*4;
                $Content .= sprintf("%${Format}sAllow:%s MetaAction:%-50s\n",
                    '- ',
                    $perm->{'Allow'},
                    $MetaAction,
                );
            }
        }
    }
    return $Content;
}

sub exportProviderUsers {
    my $Providers = LoadObjectByPath('/Providers');
    my $Content = '';
    foreach my $provider ( sort {$a->alias cmp $b->alias} @{$Providers->children}) {
        $Content .= sprintf "Provider:%s\n",$provider->get(['Alias'])->{'Alias'};
        $Content .= exportUsers($provider, 1)
    }
    return $Content;
}


sub usage {
    my $ErrorNote = shift;

    print STDERR "\n\n".$ErrorNote,"\n" if $ErrorNote;
    print <<END_USAGE;
Description:

Prints the attribute values of one or more attributes of any object. Existence
of object path and attribute is checked.

Usage:
    perl $0 [options] [flags] attribute ( {attribute} )

options:
    -passwd     database user password (optional)
    -storename  name of store (uses [storename] if not specified)
    -file       output file name
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store -file UserPermissions.txt
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
    'Message'  =>'long'
);
