use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::Alarm                     qw (Alarm);
use DE_EPAGES::Core::API::Error                     qw (ExistsError GetError);
use DE_EPAGES::Core::API::Script                    qw (RunScript);
use DE_EPAGES::Database::API::Connection            qw (RunOnStore GetCurrentDBHandle);
use DE_EPAGES::WebInterface::API::MessageCenter     qw (SynchronizeCache);
use DE_EPAGES::Core::API::Warning                   qw (Warning);
use List::Util                                      qw (shuffle);
use DE_EPAGES::Core::API::File                      qw (WriteFile);
use File::Temp                                      qw (tempdir);
use Data::Dumper;
use IO::Socket::SSL;

our $hHttpTargetList;
our $hShopList;
our $hSwitchShopList;
our $NoOutput;
our $ua;
our $timeout;

sub Main {
    local $| = 1;

    my $StoreName;
    my $Test;
    my $Time1 = time;

    GetOptions(
        'storename=s'   => \$StoreName,
        'nooutput'      => \$NoOutput,
        'test'          => \$Test,
    );

    unless (defined $StoreName) {
        usage();
    }

    RunOnStore(
        'Store' => $StoreName,
        'Sub'   => sub {

            print "### Starting Script ### \n" unless $NoOutput;

            InitializeUserAgent();
            $hHttpTargetList = GetHttpTargetList();

            GetHttpTargetResponses();
            GetHttpsTargetResponses();
            CreateLists();
            PrintData();

            print "### Finished Script ### \n" unless $NoOutput;
        }
    );

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    my $Time2 = time;
    my $Seconds = $Time2 - $Time1;
    my $LogMessage = sprintf( 'need %.2f seconds'."\n", $Seconds );

    print $LogMessage unless $NoOutput;
    exit 1;
}

# don't use net::ssl because of some segmention fault problems with current lwp version(6.05)
sub InitializeUserAgent {
    $Net::HTTPS::SSL_SOCKET_CLASS = "IO::Socket::SSL";

    use LWP::UserAgent;
    $timeout = 2;
    $ua = LWP::UserAgent->new();
    $ua->timeout($timeout);
    $ua->ssl_opts(verify_hostname => 1, SSL_verify_mode => 1);

    return;
}

sub GetHttpTargetList {
    my $SiteID;
    my $aResources;

    # TODO for which tables we need to do it also
    foreach my $tableName (qw(localizedstringattribute stringattribute)) {
        $aResources = GetResourceList($tableName);
        foreach my $resource (@{$aResources}) {
            eval {
                $SiteID = $resource->[1];

                next if (SkipResource($SiteID));

                $hHttpTargetList = ExtractTargetsFromResource($resource);
            };
            if ( ExistsError() ) {
                print GetError();
                next;
            }
        }
    }

    my $count = keys %{$hHttpTargetList};
    print "---> $count Http targets found \n" unless $NoOutput;

    return $hHttpTargetList;
}

sub ExtractTargetsFromResource {
    my ($resource) = @_;
    my $resourceValue = $resource->[0];

    while($resourceValue =~ /src=(['"])(http:\/\/.*?)\1/gix ) {
        my $HttpTargetName = $2;
        unless (defined $hHttpTargetList->{$HttpTargetName}) {
            $hHttpTargetList->{$HttpTargetName}->{'resourceIDs'} = [$resource->[2]];
            $hHttpTargetList->{$HttpTargetName}->{'shopIDs'}     = [$resource->[1]];
        }
        else {
            push(@{$hHttpTargetList->{$HttpTargetName}->{'resourceIDs'}},$resource->[2])
                unless $resource->[2] ~~ @{$hHttpTargetList->{$HttpTargetName}->{'resourceIDs'}};
            push(@{$hHttpTargetList->{$HttpTargetName}->{'shopIDs'}},$resource->[1])
                unless $resource->[1] ~~ @{$hHttpTargetList->{$HttpTargetName}->{'shopIDs'}};
        }
    }

    return $hHttpTargetList;
}

sub GetHttpTargetResponses {

    # randomise order of execution to avoid ddos someone
    foreach my $HttpTargetName( shuffle keys %{$hHttpTargetList} ) {
        my $response = GetTargetResponse($HttpTargetName);
        $hHttpTargetList->{$HttpTargetName}->{'HttpResponseCode'} = $response->code;
        $hHttpTargetList->{$HttpTargetName}->{'HttpResponseIsSuccess'} = $response->is_success ? 1 : 0;
    }

    return $hHttpTargetList;
}

sub GetHttpsTargetResponses {

    # randomise order of execution to avoid ddos someone
    foreach my $HttpTargetName( shuffle keys %{$hHttpTargetList} ) {
        my $HttpsTargetName = $HttpTargetName;
        $HttpsTargetName    =~ s/http:\/\//https:\/\//;
        my $response = GetTargetResponse($HttpsTargetName);
        $hHttpTargetList->{$HttpTargetName}->{'HttpsResponseCode'} = $response->code;
        $hHttpTargetList->{$HttpTargetName}->{'HttpsResponseIsSuccess'} = $response->is_success ? 1 : 0;
    }

    return $hHttpTargetList;
}

sub GetTargetResponse{
    my ($HttpTargetName) = @_;
    my $response;

    Alarm(
        Sub => sub {
            $response = $ua->head($HttpTargetName);
        },
        Timeout => $timeout,
        OnTimeout => sub {
        },
    );

    print 'HTTP-Statuscode: '.$response->code.' ('.$HttpTargetName.")\n" unless $NoOutput;

    return $response;
}

sub CreateLists {

    $hShopList          = TransformToOverallList();
    $hSwitchShopList    = TransformToSwitchableShopList();

    return;
}

sub PrintData {

    PrintAffectedShops();
    PrintSwitchableShops();
    PrintNotSwitchableShops();
    PrintMaySwitchableShops();
    PersistAllShops();
    PersistSwitchableShops();

    return;
}

sub TransformToOverallList {

    foreach my $HttpTargetName (keys %{ $hHttpTargetList }) {
        foreach my $ShopID (@ {$hHttpTargetList->{$HttpTargetName}->{'shopIDs'}}) {
            $hShopList->{$ShopID}->{$HttpTargetName} = $hHttpTargetList->{$HttpTargetName};

            unless (defined $hShopList->{$ShopID}->{'HttpTargetsAreValid'} && $hShopList->{$ShopID}->{'HttpTargetsAreValid'} == 0) {
                $hShopList->{$ShopID}->{'HttpTargetsAreValid'} = $hHttpTargetList->{$HttpTargetName}->{'HttpResponseIsSuccess'};
            }

            unless (defined $hShopList->{$ShopID}->{'HttpsTargetsAreValid'} && $hShopList->{$ShopID}->{'HttpsTargetsAreValid'} == 0) {
                $hShopList->{$ShopID}->{'HttpsTargetsAreValid'} = $hHttpTargetList->{$HttpTargetName}->{'HttpsResponseIsSuccess'};
            }

        }
    }

    return $hShopList;
}

sub TransformToSwitchableShopList {

    foreach my $ShopID(keys %{ $hShopList }) {
        if ($hShopList->{$ShopID}->{'HttpsTargetsAreValid'}) {
            $hSwitchShopList->{$ShopID} = $hShopList->{$ShopID};
        }
    }

    return $hSwitchShopList;
}

sub PrintMaySwitchableShops {
    my $count = 0;
    print "---> Shops may switchable\n" unless $NoOutput;
    foreach my $ShopID(keys %{ $hShopList }) {
        unless ($hShopList->{$ShopID}->{'HttpTargetsAreValid'}) {
            $count++;
            print "#### $ShopID \n" unless $NoOutput;
        }
    }
    print "---> $count Shops may switchable\n" unless $NoOutput;

    return;
}

sub PrintNotSwitchableShops {
    my $count = 0;
    print "---> Shops not switchable\n" unless $NoOutput;
    foreach my $ShopID(keys %{ $hShopList }) {
        next unless $hShopList->{$ShopID}->{'HttpTargetsAreValid'};
        unless ($hShopList->{$ShopID}->{'HttpsTargetsAreValid'}) {
            $count++;
            print "#### $ShopID \n" unless $NoOutput;
        }
    }
    print "---> $count Shops not switchable\n" unless $NoOutput;

    return;
}

sub PrintSwitchableShops {
    my $count = 0;
    print "---> Shops switchable\n" unless $NoOutput;
    foreach my $ShopID(keys %{ $hSwitchShopList }) {
        $count++;
        print "#### $ShopID \n" unless $NoOutput;
    }
    print "---> $count Shops switchable\n" unless $NoOutput;

    return;
}

sub PrintAffectedShops {
    my $count = keys %{$hShopList};
    print "---> $count Shops affected\n" unless $NoOutput;

    return;
}

sub PersistSwitchableShops {
    my $TmpDir = tempdir();

    WriteFile($TmpDir.'/switchableshops.txt', \Dumper($hSwitchShopList));

    print 'Switchable shop results can be found -> '.$TmpDir."/switchableshops.txt \n" unless $NoOutput;

    return;
}

sub PersistAllShops {
    my $TmpDir = tempdir();

    WriteFile($TmpDir.'/overall.txt', \Dumper($hShopList));

    print 'Overall results can be found -> '.$TmpDir."/overall.txt \n" unless $NoOutput;

    return;
}

# TODO: at the moment we skip all Shops which contains "Test" -> could be not what we wanted
sub SkipResource {
    my ($ShopID) = @_;

    my $sql = " SELECT so.alias FROM shop s
                INNER JOIN object so ON so.objectid=s.shopid
                WHERE so.objectid=$ShopID
                AND so.alias not like 'PreCreated%'
                AND alias not like 'WD%'
                AND alias not like 'SC%'
                AND alias not like '%Test%'";

    my $Resource = GetCurrentDBHandle()->execute($sql)->[0];

    return 0 if defined $Resource;

    return 1;
}

# Get all Shops with have a http content in resources
sub GetResourceList {
    my ($tableName) = @_;

    my $sql = " SELECT value,siteid,resource.objectid FROM $tableName resource
                INNER JOIN object ON
                resource.objectid=object.objectid AND
                (resource.value LIKE '%src=\"http://%' OR resource.value LIKE \"%src='http://%\" )";

    return GetCurrentDBHandle()->execute($sql);
}


sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -storename    storename                             ()

flags:
    -nooutput   do not print any output
    -help       show the command line options
    -test       run in test mode without resetting    (optional)

Example:
    perl $0 -storename Store -test
END_USAGE
    exit 1;
}

RunScript( 'Sub' => \&Main );
