#========================================================================================
# package      DE_EPAGES::Presentation::Servlet::JSON
# base         DE_EPAGES::Presentation::Servlet::Permission
# state        public
# description  Extends the permission servlet with processing json request of an object
#========================================================================================
package DE_EPAGES::Presentation::Servlet::JSON;
use base DE_EPAGES::Presentation::Servlet::Permission;

use strict;
use HTTP::Status qw( RC_FORBIDDEN );
use JSON qw (to_json);
use DE_EPAGES::Core::API::File qw ( ExistsFile RealPath );
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::TestParameter qw (TestObject TestString);
use DE_EPAGES::Dictionary::API::Localizer qw ( Replace );
use DE_EPAGES::Dictionary::API::XML::DictionaryImportHandler qw ( ImportDictionary );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);
use DE_EPAGES::Presentation::API::Template qw (GetTLEProcessor);
use DE_EPAGES::TLE::API::TLE qw (GetByteCodeByText);

#========================================================================================
# function     writeContentOutput
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->writeContentOutput( $rContent );
# example      $Servlet->writeContentOutput($Object->get([qw (Alias NameOrAlias ObjectID Position)]));
#----------------------------------------------------------------------------------------
# description  Writes the json content direct to the servlet output. If parameter 'callback'
#               is set, then a JSONP answer is generated.
#----------------------------------------------------------------------------------------
# input        $rContent | array or hash reference containing string values (not encoded)
#                   Only HASH and ARRAY references are allowed.
#                   Object references are not allowed.
#                   See also "JSON::encode()". | ref
#========================================================================================
sub writeContentOutput {
    my $self = shift;
    my ($rContent) = @_;

    my $JSON = to_json($rContent, {pretty => 1, indent => 2});

    my $Callback = $self->params('callback');
    if(defined $Callback){

        $self->SUPER::writeContentOutput('window["' . $Callback . '"](' . $JSON .');');
        $self->vars('ContentType', 'text/javascript');
    }
    else{
        $self->vars('ContentType', 'application/json');
        $self->SUPER::writeContentOutput($JSON);
    }
    $self->vars('ExitAfterEvent', 1);
    $self->vars('ResponseContentProcessed', 1);
    return;
}

#========================================================================================
# function     errorFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFiles = $Servlet->errorFiles( $Package, $LanguageCode );
# example      $aFiles = $Servlet->errorFiles( 'DE_EPAGES::Presentation::UI::Object', 'de' );
#----------------------------------------------------------------------------------------
# description  Returns a list of error files for a language-neutral template file.
#----------------------------------------------------------------------------------------
# input        $Package | package name of error | string
# input        $LanguageCode | language code | string
# return       $aFiles | list of error file names | array ref
#========================================================================================
sub errorFiles {
    my $self = shift;
    my ($Package, $LanguageCode) = @_;
    TestString('Package', $Package);
    TestString('LanguageCode', $LanguageCode);

    my $Cartridges = RealPath($ENV{'EPAGES_CARTRIDGES'});

    # Shared/Stores/Store/Templates/
    my $TemplateStore = LoadRootObject()->get('TemplateStore');
    my $StorePath  = join '/', RealPath($ENV{EPAGES_STORES}), $TemplateStore, 'Templates';

    my $DictionaryDirectory = '/DE_EPAGES/Dictionary';
    my @PackageNames = split /::/, $Package;
    my @Files;
    if (scalar @PackageNames > 1) {
        # /DE_EPAGES/Presentation
        my $ErrorDirectory = join '/', '', @PackageNames[0..1];

        # Dictionary Error Files for de
        # Cartridges/DE_EPAGES/Dictionary/Templates/Dictionary.de.xml
        # Shared/stores/Store/Templates/DE_EPAGES/Dictionary/Templates/Dictionary.de.xml
        # Cartridges/DE_EPAGES/Order/Templates/Dictionary.de.xml
        # Shared/stores/Store/Templates/DE_EPAGES/Order/Dictionary.de.xml
        # Cartridges/DE_EPAGES/Order/Templates/MBO/MBO-Orders.de.xml
        # Shared/stores/Store/Templates/DE_EPAGES/Order/Templates/MBO/MBO-Orders.de.xml
        @Files = (
            $Cartridges . $DictionaryDirectory . '/Templates/Dictionary',
            $Cartridges . $DictionaryDirectory . '/Templates/Error',
            $Cartridges . $ErrorDirectory . '/Templates/Dictionary',
            $Cartridges . $ErrorDirectory . '/Templates/Error',

            $StorePath . $DictionaryDirectory . '/Dictionary',
            $StorePath . $DictionaryDirectory . '/Error',
            $StorePath . $ErrorDirectory . '/Dictionary',
            $StorePath . $ErrorDirectory . '/Error',
        );
    } else {
        @Files = (
            $Cartridges . $DictionaryDirectory . '/Templates/Dictionary',
            $Cartridges . $DictionaryDirectory . '/Templates/Error',

            $StorePath . $DictionaryDirectory . '/Dictionary',
            $StorePath . $DictionaryDirectory . '/Error',
        );
    }

    return [ map { "$_.$LanguageCode.xml" } @Files ];
}

#========================================================================================
# function     errorText
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Message = $Servlet->errorText($Reason, $Package, $hVars);
#----------------------------------------------------------------------------------------
# description  Returns a localized error message.
#----------------------------------------------------------------------------------------
# input        $Reason | error code | string
# input        $Package | package of error | string
# input        $hVars | TLE variables | ref.hash
# return       $Message | error message | string
#========================================================================================
sub errorText {
    my $self = shift;
    my ($Reason, $Package, $hVars) = @_;

    my $LanguageCode = GetCodeByLanguageID( $self->languageID );
    my $Template = "{$Reason}";
    my $Dictionary = {};
    ImportDictionary($_, $Dictionary) foreach grep { ExistsFile($_) } @{$self->errorFiles($Package, $LanguageCode)};
    return '' unless defined $Dictionary->{$LanguageCode}{$Reason};

    Replace(\$Template, $Dictionary->{$LanguageCode});
    my $ByteCode = GetByteCodeByText(\$Template);
    my $TLEProcessor = GetTLEProcessor();
    $Template = $TLEProcessor->replaceTLEs($ByteCode, $hVars);
    return $Template;
}

#========================================================================================
# function     handlePermissionDenied
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->handlePermissionDenied( $Error );
#----------------------------------------------------------------------------------------
# description  Dispays the login form as a result of an unauthorized view action.
#               Triggers the action ViewLoginForm and sets the followg TLE variables:
#               <ul>
#                   <li>PermissionDenied = 1</li>
#                   <li>PermissionDeniedText = $Error->shortMessage</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
#========================================================================================
sub handlePermissionDenied {
    my $self = shift;
    my ($Error) = @_;
    TestObject( 'Error', $Error );

    $self->response->code( RC_FORBIDDEN );

    $self->writeContentOutput({
        'Errors'  => [ {
            Reason  => $Error->code,
            GUID    => $Error->guid,
            Message => $self->errorText($Error->code, $Error->package, $Error->vars)
        } ],
    });
    return;
}

#========================================================================================
# function     handleInvalidForm
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->handleInvalidForm( $Error );
# syntax       $self->handleInvalidForm( $Error, $OnChangeAction, $ErrorAction, $ErrorObjectID );
#----------------------------------------------------------------------------------------
# description  calls <function viewError> direct, if no ErrorAction defined
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
# input        $OnChangeAction | (optional) see <function DE_EPAGES::Presentation::Servlet::Permission::handleInvalidForm> | boolean
# input        $ErrorAction | (optional) error action | object
# input        $ErrorObjectID | (optional) error object id | integer
#========================================================================================
sub handleInvalidForm {
    my $self = shift;
    my ($Error, $OnChangeAction, $ErrorAction, $ErrorObjectID) = @_;

    if( $ErrorAction ){
        $self->vars('ResponseContentProcessed', 1);
        $self->SUPER::handleInvalidForm($Error, $OnChangeAction, $ErrorAction, $ErrorObjectID);
    }
    else{
        $self->viewError($Error);
        return;
    }
}

#========================================================================================
# function     viewError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->viewError( $Error );
#----------------------------------------------------------------------------------------
# description  Displays an error page by processing the page type Error with the System
#               object. Information about the error is added as TLE variables
#               <ul>
#                   <li>Error.Code - error code</li>
#                   <li>Error.ShortMessage - error message</li>
#                   <li>Error.LongMessage - error message with stack trace</li>
#                   <li>Error.Vars - additional information about the error</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Error | error object | object
#========================================================================================
sub viewError {
    my $self = shift;
    my ($Error) = @_;

    GetLog->debug('viewError: '.$Error->code);
    if ($Error->code eq 'INVALID_FORM') {
        my @Errors = map {
            {
                %$_,
                Message => $self->errorText($_->{'Reason'}, $_->{'ErrorPackage'}, $_->{'Vars'}),
            }
        } @{$self->form->formErrors};
        foreach my $error (@Errors) {
            delete $error->{'ErrorPackage'};
            delete $error->{'Error'};
        }
        $self->writeContentOutput({ 'Errors' => \@Errors });
    } else {
        $self->writeContentOutput({
            'Errors'  => [ {
                Reason  => $Error->code,
                GUID    => $Error->guid,
                Message => $self->errorText($Error->code, $Error->package, $Error->vars)
            } ],
        });
    }

    if(defined $self->params('handleashtml')) {
        $self->vars('ContentType', 'text/html');
        $self->response->header('Content-Type','text/html');
    }

    GetLog->debug('end handleError');
}

#========================================================================================
# function     processMonitorContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MonitorContentSub = $self->processMonitorContent
#----------------------------------------------------------------------------------------
# description  Disables the monitor page
#----------------------------------------------------------------------------------------
# syntax       $MonitorContentSub | undef | ref.code
#========================================================================================
sub processMonitorContent {
    my $self = shift;

    return undef;
}

1;
