#========================================================================================
# package      DE_EPAGES::ShopMailType::API::Object::ShopMailType
# state        public
# base         DE_EPAGES::MailType::API::Object::MailType
# description  object interface for ShopMailType.
#========================================================================================
package DE_EPAGES::ShopMailType::API::Object::ShopMailType;
use base DE_EPAGES::MailType::API::Object::MailType;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath );
use DE_EPAGES::Object::API::Language qw (  GetPKeyLanguageByCode );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShopMailType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShopMailType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShopMailType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopMailType = DE_EPAGES::ShopMailType::API::Object::ShopMailType->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShopMailType');
#               my $ShopMailType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShopMailType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShopMailType | new object | object
# hook         OBJ_InsertShopMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopMailTypeID - Shop mail type id    - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    if( !defined $hInfo->{'MailTypeTemplate'} ) {
        $hInfo->{'MailTypeTemplate'} = LoadRootObject()->folder('ShopMailTypeTemplates')->child($hInfo->{'Alias'});
    }
    if( !defined $hInfo->{'IsActive'} ) {
        $hInfo->{'IsActive'} = $hInfo->{'MailTypeTemplate'}->get('IsAlwaysActive');
    }
    if( !defined $hInfo->{'MailTypeTemplate'}->get('MailTypeTemplateGroup')  ) {
        $hInfo->{'MailTypeTemplate'}->set({'MailTypeTemplateGroup' => LoadObjectByPath('MailTypeTemplateGroups')->child('Misc')});
    }

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShopMailType', {'Object' => $self, 'ShopMailTypeID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopMailType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShopMailType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShopMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopMailTypeID - Shop mail type id    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShopMailType', {'Object' => $self, 'ShopMailTypeID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopMailType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShopMailType' and 'OBJ_AfterUpdateShopMailType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShopMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopMailTypeID - Shop mail type id    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShopMailType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopMailTypeID - Shop mail type id    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    # don't allow to set IsActive=0 if MailTypeTemplate has flag IsAlwaysActive and MailType has not this flag
    $hValues->{'IsActive'} = 1 if exists $hValues->{'IsActive'} and $self->get('MailTypeTemplate')->get('IsAlwaysActive') and not defined $self->get('IsAlwaysActive');

    TriggerHook('OBJ_BeforeUpdateShopMailType', {'Object'=>$self, 'Values'=> $hValues, 'ShopMailTypeID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShopMailType',  {'Object'=>$self, 'Values'=> $hValues, 'ShopMailTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ShopMailType::API::Object::ShopMailType->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Shop' => $Parent->getSite()
    };
}

#========================================================================================
# function     from
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FROM = $ShopMailType->from( $FROM );
# example      $FROM = $ShopMailType->from( $FROM);
#----------------------------------------------------------------------------------------
# description  returns FROM address string of the mail
#----------------------------------------------------------------------------------------
# input        $FROM | FROM address | string (optional)
# return       $FROM | FROM address | string
#========================================================================================
sub from {
    my $self = shift;
    my ($FROM) = @_;

    $FROM = $self->SUPER::from($FROM);
    return (defined $FROM and $FROM =~ /\S/) ? $FROM : $self->get( 'Site' )->get('MailFrom');
}

#========================================================================================
# function     to
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TO = $ShopMailType->to($TO);
# example      $TO = $ShopMailType->to($TO);
#----------------------------------------------------------------------------------------
# description  returns the TO address string of this MailType.
#               returns $TO if defined $TO.
#               returns the value of the attribute 'TO' if it is defined.
#               sets and returns the TO field to the shop attribute 'MailFrom' if the MailTypeTemplate has a TO field
#
#----------------------------------------------------------------------------------------
# input        $TO | TO address | string (optional)
# return       $TO | TO address | string
#========================================================================================
sub to {
    my $self = shift;
    my ($TO) = @_;

    $TO = $self->SUPER::to($TO);
    return $TO if defined $TO and $TO =~ /\S/ ;
    return undef unless $self->get('MailTypeTemplate')->get('HasToField');

    my $Shop = $self->get('Site');
    $TO = $self->joinMailAddress($Shop->get('NameOrAlias',$Shop->get('LanguageID')), $Shop->get('MailFrom'));
    $self->set({'TO' => $TO});

    return $TO;
}

#========================================================================================
# function     send
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Sent = $ShopMailType->send($Object, $hMailParams, $hRegionalParams, $hVars );
# example      $ShopMailType->send( $User, {
#                       HTMLAllowed => 1,
#                       TO => $User->get('EMail'),
#                   }, {
#                       Language => 'de',
#                       Locale => 'de_DE',
#                   }, { } );
#----------------------------------------------------------------------------------------
# description  Sends an e-mail object using the current shop mail type if the ShopMailType
#               is active or not.
#               Sets Header, AdditionalText and Signature if they aren't set.
#               Call afterwards SUPER::send.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# input        $hMailParams | e-mail options, see <function getMail> | ref.hash
# input        $hRegionalParams | regional options, see <function getMail> | ref.hash
# input        $hVars | TLE variables hash | ref.hash
# return       $Sent | true if the mail was actually sent | boolean
#========================================================================================
sub send {
    my $self = shift;
    my ($Object, $hMailParams, $hRegionalParams, $hVars) = @_;

    my $MailTypeTemplate = $self->get('MailTypeTemplate');
    my $LanguageID = GetPKeyLanguageByCode( $hRegionalParams->{'Language'} );

    foreach my $Attribute (qw( Header AdditionalText )) {
        if ($MailTypeTemplate->get('Has' . $Attribute)) {
            my $TLEString = defined $hVars->{$Attribute} ? $hVars->{$Attribute} : $self->get($Attribute, $LanguageID);
            $hVars->{$Attribute} = $self->replaceTLEs($Object, $TLEString, $hRegionalParams, $hVars);
        }
    }

    $hVars->{'Signature'} //= $self->getSite()->get('Signature', $LanguageID) if $MailTypeTemplate->get('HasSignature');

    return $self->SUPER::send($Object, $hMailParams, $hRegionalParams, $hVars);
}

#========================================================================================
# function     getPageType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageType = $MailType->getPageType();
#----------------------------------------------------------------------------------------
# description  Returns the page type related to the current mail type.
#----------------------------------------------------------------------------------------
# input        $Object | send mail for object | object
# return       $PageType   | PageType | PageType object
#========================================================================================

sub getPageType {
    my $self = shift;
    my ($Object) = @_;
    # test if we need a Unity PageType for the MailTypeTemplate
    if ($Object->getSite()->instanceOf('Shop') and $Object->getSite()->get('IsUnityRegistered')) {
        return $self->get('MailTypeTemplate')->getUnityPageType();
    } else {
        return $self->SUPER::getPageType();
    }
}

1;
